@@ -5,19 +5,17 @@ This makes them simpler to use but they have fewer features.
 """
 from __future__ import annotations
 
-__version__ = "2.1.0"
+__version__ = "2.2.0"
 
 import reprlib
-import warnings
 from typing import (
     Any,
     Callable,
     ClassVar,
     DefaultDict,
-    Dict,
     Iterable,
     Iterator,
-    List,
+    MutableMapping,
     Optional,
     Sequence,
     Type,
@@ -30,33 +28,15 @@ T = TypeVar("T")
 
 
 ComponentDictObserver = Callable[["ComponentDict", Type[T], Optional[T], Optional[T]], None]
+_ComponentDictObserver = Callable[["ComponentDict", Optional[T], Optional[T]], None]
 
 
-def abstract_component(cls: Type[T]) -> Type[T]:
+def abstract_component(cls: type[T]) -> type[T]:
     """Register class `cls` as an abstract component and return it.
 
     Subclasses of this `cls` will now use `cls` as the key when being accessed in :any:`ComponentDict`.
     This means that ComponentDict can only hold one unique instance of this subclass.
 
-    .. deprecated:: 2.0
-
-        This method of handling abstract components was deemed unnecessary.
-        Instead a new component should be made to hold the base class, for example::
-
-            >>> from tcod.ec import ComponentDict
-            >>> import attrs
-            >>> @attrs.define
-            ... class Base:
-            ...     pass
-            >>> @attrs.define
-            ... class Derived(Base):
-            ...     pass
-            >>> @attrs.define
-            ... class Abstract(Base):
-            ...     value: Base
-            >>> ComponentDict([Abstract(Derived())])
-            ComponentDict([Abstract(value=Derived())])
-
     Example::
 
         >>> from tcod.ec import ComponentDict, abstract_component
@@ -84,22 +64,20 @@ def abstract_component(cls: Type[T]) -> Type[T]:
         >>> entity[Base]
         Derived()
     """
-    warnings.warn(
-        "The abstract_component decorator is deprecated."
-        "  To store a derived class: make a new component which stores the base class and holds the derived object.",
-        FutureWarning,
-        stacklevel=2,
-    )
     cls._COMPONENT_TYPE = cls  # type: ignore[attr-defined]
     return cls
 
 
-class ComponentDict:
+# Maybe needs PEP 695: https://peps.python.org/pep-0695/
+class ComponentDict(MutableMapping[Type[Any], Any]):
     """A dictionary of component instances, addressed with their class as the key.
 
     This class implements the idea of a ``Dict[Type[T], T]`` type-hint.
     This allows adding data and behavior to an object without needing to define which classes the object holds ahead of time.
 
+    For equality and hashing ComponentDict's use identity like a standard :any:`object` rather than like a :any:`dict`.
+    This means two different ComponentDict's are considered unique even if they have the same exact components.
+
         >>> import attrs
         >>> from tcod.ec import ComponentDict
         >>> @attrs.define
@@ -123,6 +101,24 @@ class ComponentDict:
         >>> ComponentDict([Position(1, 2), Position(3, 4)])  # The same component always overwrites the previous one.
         ComponentDict([Position(x=3, y=4)])
 
+    Observers can be used to track the assignment of components.
+    These work best when the components are immutable and frozen.
+
+        >>> @attrs.define(frozen=True)
+        ... class Position():
+        ...     x: int = 0
+        ...     y: int = 0
+        >>> def track_position(entity: ComponentDict, new_pos: Position | None, old_pos: Position | None) -> None:
+        ...     print(f"Moved: {old_pos} -> {new_pos}")
+        >>> entity = ComponentDict(components=[Position()], observers={Position: [track_position]})
+        Moved: None -> Position(x=0, y=0)
+        >>> entity
+        ComponentDict([Position(x=0, y=0)], observers=...)
+        >>> entity[Position] = Position(1, 2)
+        Moved: Position(x=0, y=0) -> Position(x=1, y=2)
+        >>> del entity[Position]
+        Moved: Position(x=1, y=2) -> None
+
     Custom functions can be added to the class variable :any:`global_observers` to trigger side-effects on component assignment.
     This can be used to register components to a global system, handle save migration, or other effects::
 
@@ -141,14 +137,17 @@ class ComponentDict:
         >>> del entity[Position]
         Position: Position(x=1, y=2) -> None
         >>> ComponentDict.global_observers.remove(print_changes)
+
+    .. versionchanged:: 2.2
+        Is now a :any:`collections.abc.MutableMapping` and has all of the relevant methods such as ``.values()``.
     """
 
-    __slots__ = ("_components", "__weakref__")
+    __slots__ = ("_components", "observers", "__weakref__")
 
-    _components: Dict[Type[Any], Any]
+    _components: dict[type[Any], Any]
     """The actual components stored in a dictionary.  The indirection is needed to make type hints work."""
 
-    global_observers: ClassVar[List[ComponentDictObserver[Any]]] = []
+    global_observers: ClassVar[list[ComponentDictObserver[Any]]] = []
     '''A class variable list of functions to call with component changes.
 
     Unpickled and copied objects are observed as if their components are newly created.
@@ -176,7 +175,29 @@ class ComponentDict:
         Use :any:`clear` when you are finished with an entity and want its components observed as being deleted.
     '''
 
-    def __init__(self, components: Iterable[object] = ()) -> None:
+    def __init__(
+        self,
+        components: Iterable[object] = (),
+        observers: dict[type[Any], list[_ComponentDictObserver[Any]]] | None = None,
+    ) -> None:
+        """Initialize a ComponentDict.
+
+        Args:
+            components: An iterable of component objects.
+                These are assigned by their type.
+                If any components share the same type then the latter components overwrite the previous ones.
+            observers: A dictionary of component observers.
+                This is held by reference, so the same dict my be assigned to multiple ComponentDict's.
+                Observers are called with the `components` used to create the ComponentDict.
+
+        .. versionchanged:: 2.2
+            Added `observers` parameter.
+        """
+        self.observers: dict[type[Any], list[_ComponentDictObserver[Any]]] = {} if observers is None else observers
+        """A dictionary of a list of component observers.
+
+        .. versionadded:: 2.2
+        """
         self._components = {}
         self.set(*components)
 
@@ -188,7 +209,7 @@ class ComponentDict:
             real_key is key
         ), f"{key!r} is a child of an abstract component and can only be accessed with {real_key!r}."
 
-    def set(self, *components: object) -> Self:  # type: ignore[valid-type]  # Needs mypy >0.991
+    def set(self, *components: object) -> Self:
         """Assign or replace the components of this entity and return self.
 
         .. versionchanged:: 1.1
@@ -198,13 +219,7 @@ class ComponentDict:
             self[getattr(component, "_COMPONENT_TYPE", component.__class__)] = component
         return self
 
-    def get(self, key: Type[T]) -> Optional[T]:
-        """Return a component, or None if it doesn't exist."""
-        if __debug__:
-            self.__assert_key(key)
-        return self._components.get(key)  # Cast to Optional[T].
-
-    def __getitem__(self, key: Type[T]) -> T:
+    def __getitem__(self, key: type[T]) -> T:
         """Return a component of type, raises KeyError if it doesn't exist."""
         if __debug__:
             self.__assert_key(key)
@@ -213,7 +228,7 @@ class ComponentDict:
             return value  # type: ignore[no-any-return]  # Cast to T.
         return self.__missing__(key)
 
-    def __setstate__(self, state: Any | Dict[str, Any]) -> None:
+    def __setstate__(self, state: Any | dict[str, Any]) -> None:
         """Unpickle instances from 1.0 or later, or complex subclasses from 2.0 or later.
 
         Component classes can change between picking and unpickling, and component order should be preserved.
@@ -222,23 +237,30 @@ class ComponentDict:
         The unpickled object acts as if its components are newly assigned to it when observed.
         """
         # Normalize attrs handling.
-        dict_state: Dict[str, Any] = state[1] if isinstance(state, tuple) else state
+        dict_state: dict[str, Any] = state[1] if isinstance(state, tuple) else state
 
         components: Iterable[object] = dict_state["_components"]
         del dict_state["_components"]
         if isinstance(components, dict):  # Convert v1.1 _components attribute into instance list.
             components = components.values()
 
+        observers = {}
+        if "observers" in dict_state:  # Missing from old objects.
+            observers = dict_state["observers"]
+            del dict_state["observers"]
+
         for attr, value in dict_state.items():
             setattr(self, attr, value)
 
-        # Unpack components with side-effects.
+        # Unpack components with global side-effects.
+        self.observers = {}
         self._components = {}
         self.set(*components)
+        self.observers = observers
 
-    def __getstate__(self) -> Dict[str, Any]:
+    def __getstate__(self) -> dict[str, Any]:
         """Pickle this instance.  Any subclass slots and dict attributes will also be saved."""
-        state: Dict[str, Any] = {}
+        state: dict[str, Any] = {}
         for cls in self.__class__.__mro__:
             for attr in getattr(cls, "__slots__", ()):
                 if not hasattr(self, attr):
@@ -252,8 +274,8 @@ class ComponentDict:
         state["_components"] = tuple(state["_components"].values())
         return state
 
-    def __missing__(self, key: Type[T]) -> T:
-        '''Called when a key is missing.  Raises KeyError with the missing key.
+    def __missing__(self, key: type[T]) -> T:
+        '''Raise KeyError with the missing key.  Called when a key is missing.
 
         Example::
 
@@ -267,24 +289,29 @@ class ComponentDict:
         '''
         raise KeyError(key)
 
-    def __setitem__(self, key: Type[T], value: T) -> None:
+    def __setitem__(self, key: type[T], value: T) -> None:
         """Set or replace a component."""
         valid_key = getattr(value, "_COMPONENT_TYPE", value.__class__)
         if key is not valid_key:
-            raise TypeError(f"{value!r} is being assigned to {key!r} but it belongs to {valid_key!r} instead!")
+            msg = f"{value!r} is being assigned to {key!r} but it belongs to {valid_key!r} instead!"
+            raise TypeError(msg)
         old_value = self._components.get(key)
         self._components[key] = value
-        for observer in self.global_observers:
-            observer(self, key, value, old_value)
+        for global_observer in self.global_observers:
+            global_observer(self, key, value, old_value)
+        for local_observer in self.observers.get(key, ()):
+            local_observer(self, value, old_value)
 
-    def __delitem__(self, key: Type[object]) -> None:
+    def __delitem__(self, key: type[object]) -> None:
         """Delete a component."""
         old_value = self._components[key]
         del self._components[key]
-        for observer in self.global_observers:
-            observer(self, key, None, old_value)
+        for global_observer in self.global_observers:
+            global_observer(self, key, None, old_value)
+        for local_observer in self.observers.get(key, ()):
+            local_observer(self, None, old_value)
 
-    def __contains__(self, keys: Type[object] | Iterable[Type[object]]) -> bool:
+    def __contains__(self, keys: type[object] | Iterable[type[object]]) -> bool:  # type: ignore[override]
         """Return true if the types of component exist in this entity.  Takes a single type or an iterable of types.
 
         .. versionchanged:: 1.2
@@ -301,22 +328,22 @@ class ComponentDict:
         """Return the number of components contained in this object."""
         return len(self._components)
 
-    def __iter__(self) -> Iterator[Type[Any]]:
+    def __iter__(self) -> Iterator[type[Any]]:
         """Iterate over the keys of this container."""
         return iter(self._components)
 
+    # Use identity comparison and hashing.
+    __hash__ = object.__hash__
+    __eq__ = object.__eq__
+    __ne__ = object.__ne__
+
     @reprlib.recursive_repr()
     def __repr__(self) -> str:
         """Return the representation of this ComponentDict."""
-        return f"{self.__class__.__name__}([{', '.join(repr(component) for component in self._components.values())}])"
-
-    def clear(self) -> None:
-        """Remove all components from this container.
-
-        .. versionadded:: 2.0
-        """
-        for component_cls in list(self):
-            del self[component_cls]
+        params = [f"[{', '.join(repr(component) for component in self._components.values())}]"]
+        if self.observers:
+            params.append(f"observers={self.observers!r}")
+        return f"{self.__class__.__name__}({', '.join(params)})"
 
 
 class Composite:
@@ -363,14 +390,19 @@ class Composite:
         >>> spell
         Composite(['fire', 'effect', 20, Square(range=3)])
 
-    .. versionadded:: Unreleased
+    .. versionadded:: 2.1
     """
 
     __slots__ = ("_components", "__weakref__")
 
-    _components: Dict[Type[Any], List[Any]]
+    _components: dict[type[Any], list[Any]]
 
     def __init__(self, components: Iterable[object] = ()) -> None:
+        """Initialize a Composite object from an iterable of components.
+
+        Args:
+            components: An iterable of objects.
+        """
         self._components = DefaultDict(list)
         for obj in components:
             self.add(obj)
@@ -400,7 +432,7 @@ class Composite:
         if object in self._components:
             del self[object]
 
-    def __getitem__(self, key: Type[T]) -> Sequence[T]:
+    def __getitem__(self, key: type[T]) -> Sequence[T]:
         """Return a sequence of all instances of `key`.
 
         If no instances of `key` are stored then return an empty sequence.
@@ -411,20 +443,20 @@ class Composite:
         """
         return self._components.get(key, ())
 
-    def __setitem__(self, key: Type[T], values: Iterable[T]) -> None:
+    def __setitem__(self, key: type[T], values: Iterable[T]) -> None:
         """Replace all instances of `key` with the instances of `values`."""
         del self[key]
         for obj in values:
             self.add(obj)
 
-    def __delitem__(self, key: Type[object]) -> None:
+    def __delitem__(self, key: type[object]) -> None:
         """Remove all instances of `key` if they exist."""
         if key not in self._components:
             return
         for obj in list(self._components[key]):
             self.remove(obj)
 
-    def __contains__(self, keys: Type[object] | Iterable[Type[object]]) -> bool:
+    def __contains__(self, keys: type[object] | Iterable[type[object]]) -> bool:
         """Return true if all types or sub-types of `keys` exist in this entity.
 
         Takes a single type or an iterable of types.
@@ -433,7 +465,7 @@ class Composite:
             keys = (keys,)
         return all(key in self._components for key in keys)
 
-    def __setstate__(self, state: Dict[str, Any]) -> None:
+    def __setstate__(self, state: dict[str, Any]) -> None:
         """Unpickle instances of this object.
 
         Any class changes in pickled components will be reflected correctly.
@@ -449,9 +481,9 @@ class Composite:
         for component in components:
             self.add(component)
 
-    def __getstate__(self) -> Dict[str, Any]:
+    def __getstate__(self) -> dict[str, Any]:
         """Pickle this instance.  Any subclass slots and dict attributes will also be saved."""
-        state: Dict[str, Any] = {}
+        state: dict[str, Any] = {}
         for cls in self.__class__.__mro__:
             for attr in getattr(cls, "__slots__", ()):
                 if not hasattr(self, attr):
